@@ -618,10 +618,12 @@ switch_to_compatible_user_session (GdmManager *manager,
 
         if (existing_session != NULL) {
                 ssid_to_activate = gdm_session_get_session_id (existing_session);
-                res = activate_session_id (manager, seat_id, ssid_to_activate);
-                if (! res) {
-                        g_debug ("GdmManager: unable to activate session: %s", ssid_to_activate);
-                        goto out;
+                if (seat_id != NULL) {
+                        res = activate_session_id (manager, seat_id, ssid_to_activate);
+                        if (! res) {
+                                g_debug ("GdmManager: unable to activate session: %s", ssid_to_activate);
+                                goto out;
+                        }
                 }
 
                 res = session_unlock (manager, ssid_to_activate);
@@ -781,8 +783,10 @@ gdm_manager_handle_register_display (GdmDBusManager        *manager,
         if (session != NULL) {
                 GPid pid;
 
-                if (x11_display_name != NULL)
+                if (x11_display_name != NULL) {
                         g_object_set (G_OBJECT (session), "display-name", x11_display_name, NULL);
+                        g_object_set (G_OBJECT (display), "x11-display-name", x11_display_name, NULL);
+                }
 
                 /* FIXME: this should happen in gdm-session.c when the session is opened
                  */
@@ -1327,6 +1331,7 @@ set_up_automatic_login_session (GdmManager *manager,
                                 GdmDisplay *display)
 {
         GdmSession *session;
+        char       *display_session_type = NULL;
         gboolean is_initial;
 
         /* 0 is root user; since the daemon talks to the session object
@@ -1334,8 +1339,14 @@ set_up_automatic_login_session (GdmManager *manager,
          */
         session = create_embryonic_user_session_for_display (manager, display, 0);
 
-        g_object_get (G_OBJECT (display), "is-initial", &is_initial, NULL);
-        g_object_set (G_OBJECT (session), "display-is-initial", is_initial, NULL);
+        g_object_get (G_OBJECT (display),
+                      "is-initial", &is_initial,
+                      "session-type", &display_session_type,
+                      NULL);
+
+        g_object_set (G_OBJECT (session),
+                      "display-is-initial", is_initial,
+                      NULL);
 
         g_debug ("GdmManager: Starting automatic login conversation");
         gdm_session_start_conversation (session, "gdm-autologin");
@@ -1417,6 +1428,7 @@ on_display_status_changed (GdmDisplay *display,
                                                 set_up_greeter_session (manager, display);
                                         }
                                 }
+                                g_free (session_class);
                         }
 
                         if (status == GDM_DISPLAY_MANAGED) {
@@ -1722,6 +1734,15 @@ remove_user_session (GdmManager *manager,
         }
 }
 
+static void
+on_session_start_failed (GdmSession *session,
+                         const char *service_name,
+                         GdmManager *manager)
+{
+        g_debug ("GdmManager: session failed to start");
+        remove_user_session (manager, session);
+}
+
 static void
 on_user_session_exited (GdmSession *session,
                         int         code,
@@ -2024,6 +2045,10 @@ create_embryonic_user_session_for_display (GdmManager *manager,
         char       *display_auth_file = NULL;
         char       *display_seat_id = NULL;
         char       *display_id = NULL;
+#ifdef ENABLE_WAYLAND_SUPPORT
+        char       *display_session_type = NULL;
+        gboolean    greeter_is_wayland;
+#endif
 
         g_object_get (G_OBJECT (display),
                       "id", &display_id,
@@ -2032,6 +2057,9 @@ create_embryonic_user_session_for_display (GdmManager *manager,
                       "remote-hostname", &remote_hostname,
                       "x11-authority-file", &display_auth_file,
                       "seat-id", &display_seat_id,
+#ifdef ENABLE_WAYLAND_SUPPORT
+                      "session-type", &display_session_type,
+#endif
                       NULL);
         display_device = get_display_device (manager, display);
 
@@ -2093,6 +2121,10 @@ create_embryonic_user_session_for_display (GdmManager *manager,
                           "session-started",
                           G_CALLBACK (on_user_session_started),
                           manager);
+        g_signal_connect (session,
+                          "session-start-failed",
+                          G_CALLBACK (on_session_start_failed),
+                          manager);
         g_signal_connect (session,
                           "session-exited",
                           G_CALLBACK (on_user_session_exited),
@@ -2107,6 +2139,12 @@ create_embryonic_user_session_for_display (GdmManager *manager,
                                 g_object_ref (session),
                                 (GDestroyNotify)
                                 clean_embryonic_user_session);
+
+#ifdef ENABLE_WAYLAND_SUPPORT
+        greeter_is_wayland = g_strcmp0 (display_session_type, "wayland") == 0;
+        g_object_set (G_OBJECT (session), "ignore-wayland", !greeter_is_wayland, NULL);
+#endif
+
         return session;
 }
 
